# TweeLoc
'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

	Copyright Paraskevopoulos, Pellegrini, Palpanas.


	This software cannot be used for commercial purposes.

	You may use the software as is, only for research purposes.
	You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

	Pavlos Paraskevopoulos, Themis Palpanas.
	Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
	http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

	Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
	http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

TweeLoc is written with Python version 2.7.

## Prerequisites

If you want to run the algorithmic part of TweeLoc, these are the dependencies you must install:

```
    + geopy
    + numpy
    + matplotlib
    + tweepy
    + scikit-learn
```

Regarding the visualization part you have to install the following dependencies:
```
    + geoplotlib
    + scikit
    + PySide
    + pyglet
```

Algorithmic and Visualization parts of the sofware can be run separately, though the Visualization needs data
produced in the algorithmic part to plot the results.


## Installing

Authors suggestions is to create a virtual environment using virtualenv.
Otherwise, a user can install all the dependencies manually. The version
of geoplotlib used is a custom version, please remember to copy the
geoplotlib folder to the right path to python's dependencies (site-packages
or dist-packages).

In order to install the dependencies in virtualenv, copy the TweeLoc folder
inside the virtualenv folder and run the file
```
    ./setup.sh
```

## Running the tests

To start the program in the "offline" mode, just type the command:
```
   python[2.7] tweeloc.py -t offline
```
Offline mode basically means that the computation is performed with an existing
dataset of tweets. Tweets are stored in Json files, with the explicit extension
.json.

To run the program in the "online" mode, type:
```
   python[2.7] tweeloc.py -t online
```
In the online version tweets are downloaded directly from Twitter tweet stream,
and analyzed at each window slide forward. Run this mode in case you want to
test it in a real time case.

Before either the offline or online mode, check if the settings.txt file is set
correctly. For more information, see the Appendix.

### Results

The results of each online or offline computation are stored inside the folder
Run_[date-time] of the run execution. Inside the folder are contained other subfolders
for each result and info stored. Plots contains the plots of the performances, Predictions
the tweets for which a location has been assigned by TweeLoc, Results the text written
performances for each run for each threshold/percentage. Finally, Visualization_data
contains all the data used in the Visualization mode.

### Example test
Software is provided with an example of a tweet dataset of three days. To run it, just
type the commands above.

## Run Visualization

To run the Visualization of the data produced by the computation, run
```
   python[2.7] visualization.py normal Run_[date-time]
```
to plot different statistics of the data, such Train Heatmap, Prediction Heatmap, Differential Heatmap
and the distribution of the tweets.

Instead, with the command:
```
   python[2.7] visualization.py stream Run_[date-time]
```
visualization shows a stream of tweets predicted by TweeLoc during the computation, assigning
a coordinate latitude, longitude to tweets without geolocation enabled.


# Appendix

## Settings file

#### Basic settings
input_folder=       Folder which contains the files with the tweets downloaded from Tweet stream. File must be json type.

start_datetime=     Starting date and time of the computation in the form dd-mm-yy hh:mm:ss (01-Jun-2016 14:00:00).
                    Remember that the results will be taken into consideration after one day (02-Jun-2016 14:00:00)

stop_datetime=      Stopping date and time of the computation (same form as above)

input_l1=           Input of the cities coordinates you want to test. The coordinates for the box which contains the city
                    is constructed from the north-west point of the box, then the measure in meters of the x_axis and the
                    y_axis, in the form: CityName:LatOfNorthWestPoint,LonOfNorthWestPoint,x_side,y_side. To separate one city
                    from another, use the pipe (|). e.g. Roma:41.992099,12.376495,23000,25000|Milano:45.537070,9.077824,15000,16000


#### Algorithm settings
l2_square_side=     Size in meters of the side of the square inside the city.

window_size=        Number of timeslots that compose the window.

timeslot_size=      Size in minutes of one timeslot.

n_runs=             Number of times you want to run the algorithm. Final results will be averaged among the runs.

train_percentage=   Value from 0.1 to 0.99 that states the percentage of tweets for the training set.

languages_filter=   Tweets with different language from the indicates one are discarded. If you wan to use every language, put none.

stop_words=         Words to be filtered from the text of the tweets, separated by comma.

thresholds_list=    List of the thresholds you want to use. The number specify the percentage of the original threshold to add to it.
                    0 means original threshold. Always put 0. (e.g 0,20,10,-10,-20,no)

keywords_percentage=    List of the percentage of the keywords you want to keep from the text of the tweet. e.g
						(10,20,30,40,50,60,70,80,90,100)


#### Test type

l1_correlation=     Boolean parameter to set if you want or not to use the linear regression for the first level prediction (true or false)

l2_correlation=     Boolean parameter to set if you want or not to use the linear regression for the second level prediction (true or false)


#### Thresholds file

thresholds_file=	Path to the file containing the thresholds of a previous run. To be able to use the thresholds in the file, the number
					of keywords percentage adn the number of timeslots per day must be the same. If you don't want to use the thresholds,
					put 'none'



#### Visualization

stream=true			Set flag to true/false to store predictions of the tweets without any position.


#### Twitter credential (Used just in the online mode)
consumer_key=       Consumer key to access the Twitter streaming
consumer_secret=        Consumer secret to access the Twitter streaming
access_token_key=       Access token key to access the Twitter streaming
access_token_secret=    Access token secret to access the Twitter streaming

#### Bounding Box (Used just in the online mode)
bounding_box=       Cooedinate of the bounding box to download the tweets from. e.g. bounding box of Italy 6.911300,36.406382,18.136126,47.363242

