'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

	Copyright Paraskevopoulos, Pellegrini, Palpanas.


	This software cannot be used for commercial purposes.

	You may use the software as is, only for research purposes.
	You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

	Pavlos Paraskevopoulos, Themis Palpanas.
	Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
	http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

	Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
	http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

from geoplotlib.layers import BaseLayer, HotspotManager
from geoplotlib.core import BatchPainter

import numpy as np


class ComparisonLayer(BaseLayer):
	def __init__(self, data, size, communicator):
		self.data = data
		self.size = size
		self.communicator = communicator
		self.incremental_distance = [0, 0]
		try:
			self.incremental_distance[0] = np.linalg.norm(np.array(self.data[0]) - np.array(self.data[1]))
			self.incremental_distance[1] = np.linalg.norm(np.array(self.data[0]) - np.array(self.data[2]))
		except:
			self.incremental_distance = [0, 0]

	def invalidate(self, proj):
		self.hotspot = HotspotManager()
		self.painter = BatchPainter()
		color = [0, 0, 0, 220]
		xv = []
		yv = []
		self.lons = []
		self.lats = []
		try:
			for s in range(0, 3):
				self.lons.append(self.data[s][1])
				self.lats.append(self.data[s][0])
				color[s] = 255
				x, y = proj.lonlat_to_screen(np.array(self.data[s][1]), np.array(self.data[s][0]))
				xv.append(x)
				yv.append(y)
				self.painter.set_color(color)
				self.painter.points(x, y, self.size, True)
				color = [0, 0, 0, 220]

			#proj.fit(BoundingBox.from_points(lons=np.array(self.lons), lats=np.array(self.lats)))
			labels = ["Original Datapoint", "TweeLoc prediction", "QL prediction"]
			self.painter.set_color([0, 0, 0, 220])

			self.painter.labels(np.array(xv), np.array(yv), labels, font_size=16, anchor_x='center',
								anchor_y='top')
		except:
			print "No more tweets for comparison in this timeslot."
		#print "Incremental distance: {}".format(self.incremental_distance)

	def draw(self, proj, mouse_x, mouse_y, ui_manager):
		self.painter.batch_draw()
		self.communicator.add_infobar(ui_manager, "CO")
		try:
			tweeloc_distance = np.linalg.norm(np.array(self.data[0]) - np.array(self.data[1]))
			ql_distance = np.linalg.norm(np.array(self.data[0]) - np.array(self.data[2]))
		except:
			tweeloc_distance = 0
			ql_distance = 0
		self.communicator.add_statusbar(ui_manager, "TweeLoc distance: {0}   QL distance: {1}   Incremental [TweeLoc, QL]: {2}, {3}".format(tweeloc_distance, ql_distance, self.incremental_distance[0], self.incremental_distance[1]))

	def update_data(self, data):
		self.data = data
		if len(self.data) != 0:
			self.incremental_distance[0] += np.linalg.norm(np.array(self.data[0]) - np.array(self.data[1]))
			self.incremental_distance[1] += np.linalg.norm(np.array(self.data[0]) - np.array(self.data[2]))

	def on_key_release(self, key, modifiers):
		self.communicator.on_key_release(key, "comparison")



