'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

	Copyright Paraskevopoulos, Pellegrini, Palpanas.


	This software cannot be used for commercial purposes.

	You may use the software as is, only for research purposes.
	You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

	Pavlos Paraskevopoulos, Themis Palpanas.
	Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
	http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

	Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
	http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

from geoplotlib.layers import BaseLayer
from geoplotlib import colors
from geoplotlib.layers import HotspotManager
from geoplotlib.core import BatchPainter
from geoplotlib.utils import BoundingBox
from utilities import *
import os
import glob
import numpy as np
import pyglet


class OnlinePredictionLayer(BaseLayer):
	def __init__(self, bw, cmap='hot', scaling='sqrt', alpha=220,
				cut_below=None, clip_above=None, binsize=1, cmap_levels=20, data_q=None, folder=None):
		self.bw = bw
		self.folder = folder+'/'
		self.cmap = colors.ColorMap(cmap, alpha=alpha, levels=cmap_levels)
		self.scaling = scaling
		self.cut_below = cut_below
		self.clip_above = clip_above
		self.binsize = binsize
		self.data_q = data_q
		self.tweets = None
		self.tweet = None
		self.sq_id = -1
		self.loc_sq = ''
		self.loc_coordinates = {}
		refer_file = glob.glob(self.folder + "Visualization_data/Squares_data/*")
		ts_files_list = sorted(glob.glob(refer_file[0] + "/*"))
		self.ts_files = []
		for ts_file in ts_files_list:
			dir_name = os.path.basename(ts_file)
			if not dir_name == "columns.txt" and not dir_name == "rows.txt":
				self.ts_files.append(dir_name)
		self.current_file = 0

		path_to_locs = glob.glob(self.folder + "Visualization_data/Squares_data/*")
		loc_names = []
		for path in path_to_locs:
			loc_names.append(os.path.basename(path))
		for loc_name in loc_names:
			cols = {}
			with open(self.folder + "Visualization_data/Squares_data/" + str(loc_name) + "/columns.txt") as col_file:
				for line in col_file:
					tokens = line.split("\t")
					id = int(tokens[0])
					sx = float(tokens[1])
					dx = float(tokens[2])
					cols.update({id: {"sx": sx, "dx": dx}})
			rows = {}
			with open(self.folder + "Visualization_data/Squares_data/" + str(loc_name) + "/rows.txt") as row_file:
				for line in row_file:
					tokens = line.split("\t")
					id = int(tokens[0])
					up = float(tokens[1])
					dw = float(tokens[2])
					rows.update({id: {"up": up, "dw": dw}})
			self.loc_coordinates.update({loc_name: [cols, rows]})
		self.hotspots = HotspotManager()
		self.keywords = {}
		self.update_keywords()


	def draw(self, proj, mouse_x, mouse_y, ui_manager):
		self.painter = BatchPainter()
		lon = []
		lat = []
		if self.data_q.qsize() > 0:
			tmp = self.data_q.get()
			if type(tmp) is list:
				self.sq_id = -1
				self.tweet = None
				self.tweets = tmp
				for tweet in self.tweets:
					lon.append(tweet.position['lon'])
					lat.append(tweet.position['lat'])
				proj.fit(BoundingBox.from_points(lons=np.array(lon),
												 lats=np.array(lat)))
				xvt, yvt = proj.lonlat_to_screen(np.array(lon), np.array(lat))
				# self.painter.set_color([0, 0, 0])
				self.painter.points(xvt, yvt, rounded=True, point_size=10)
			elif isinstance(tmp, Tweet):
				self.tweets = None
				self.tweet = tmp
				lon.append(self.tweet.position['lon'])
				lat.append(self.tweet.position['lat'])
				xvt, yvt = proj.lonlat_to_screen(np.array(lon), np.array(lat))
				self.painter.circle_filled(xvt, yvt, r=10)

		elif self.tweets:
			self.sq_id = -1
			for tweet in self.tweets:
				lon.append(tweet.position['lon'])
				lat.append(tweet.position['lat'])
			xvt, yvt = proj.lonlat_to_screen(np.array(lon), np.array(lat))
			# self.painter.set_color([0, 0, 0])
			self.painter.points(xvt, yvt, rounded=True, point_size=10)
		elif self.tweet:
			lon_t = []
			lat_t = []
			for loc_name, loc_sq_ids in self.loc_coordinates.items():
				for c_id, c_limits in loc_sq_ids[0].items():
					for r_id, r_limits in loc_sq_ids[1].items():
						if (c_limits['sx'] <= self.tweet.position['lon'] < c_limits['dx']) and (
										r_limits['dw'] <= self.tweet.position['lat'] < r_limits['up']):
							self.loc_sq = loc_name
							sq_id = index_to_id(r_id, c_id)
							self.sq_id = sq_id
							lon = np.array([c_limits['sx'], c_limits['dx']])
							lat = np.array([r_limits['dw'], r_limits['up']])
							x, y = proj.lonlat_to_screen(lon, lat)
							w = x.max() - x.min()
							h = y.max() - y.min()
							w = np.ceil(w / 2) * 2
							h = np.ceil(h / 2) * 2
							l = max(w, h)
							n_x = [x.min(), x.min() + l, x.min() + l, x.min()]
							n_y = [y.min() + l, y.min() + l, y.min(), y.min()]
							self.painter.set_color([0, 0, 0])
							self.painter.linestrip(n_x, n_y, closed=True)
							self.hotspots.add_poly(n_x, n_y, self.keywords[loc_name][sq_id])
							self.painter.set_color('b')
							lon_t.append(self.tweet.position['lon'])
							lat_t.append(self.tweet.position['lat'])
							xvt, yvt = proj.lonlat_to_screen(np.array(lon_t), np.array(lat_t))
							self.painter.circle_filled(xvt, yvt, r=10)
							picked = self.hotspots.pick(mouse_x, mouse_y)
							if picked:
								ui_manager.tooltip(picked)
		self.painter.batch_draw()

	def on_mouse_release(self, proj, mouse_x,  mouse_y, modifiers):
		if self.sq_id == -1:
			return
		p_lat, p_lon = proj.screen_to_latlon(mouse_x, mouse_y)
		loc_sq_ids = self.loc_coordinates[self.loc_sq]
		for c_id, c_limits in loc_sq_ids[0].items():
			for r_id, r_limits in loc_sq_ids[1].items():
				if (c_limits['sx'] <= p_lon < c_limits['dx']) and (r_limits['dw'] <= p_lat < r_limits['up']):
					sq_id = index_to_id(r_id, c_id)
					if sq_id == self.sq_id:
						if os.path.exists(self.folder + "Visualization_data/Squares_data/" + self.loc_sq + "/Time_series/" + str(
								self.sq_id) + ".png"):
							img_stream = open(
								self.folder + "Visualization_data/Squares_data/" + self.loc_sq + "/Time_series/" + str(self.sq_id) + ".png",
								"rb")
							image = pyglet.image.load('time_series.png', file=img_stream)
							window = pyglet.window.Window(width=image.width, height=image.height)
							window.set_caption("Time series")

							@window.event
							def on_draw():
								window.clear()
								image.blit(0, 0)

							pyglet.app.run()



	def update_keywords(self):
		self.keywords = {}
		for loc_name in self.loc_coordinates.keys():
			self.keywords.update({loc_name: {}})
			sq_files = glob.glob(
				self.folder + "Visualization_data/Squares_data/" + loc_name + "/" + self.ts_files[self.current_file] + "/*")
			for sq_file_path in sq_files:
				sq_id = os.path.basename(sq_file_path)
				id = int(sq_id[:-4])
				with open(sq_file_path, 'r') as sq_file:
					for line in sq_file:
						s_line = line.split("=")
						if s_line[0] == "keywords":
							keywords_vect = s_line[1].split(" ")
							keywords = "keywords: "
							for w in keywords_vect:
								if not w == '':
									keywords += w + ", "
							self.keywords[loc_name].update({id: keywords[:-2]})
							break


class OnlinePredictionLayer2(BaseLayer):
	def __init__(self, coordinates):
		self.coordinates = coordinates
		self.tweets = []

	def invalidate(self, proj):
		self.painter = BatchPainter()
		self.hotspots = HotspotManager()
		lon_t = []
		lat_t = []
		if len(self.tweets) == 1:
			for name, sq_ids in self.coordinates.items():
				c_id = -1
				for c in sq_ids[0]:
					r_id = 0
					c_id += 1
					for r in sq_ids[1]:
						if c[0] < self.tweets[0].position['lon'] < c[1] and r[1] < self.tweets[0].position['lat'] < r[0]:
							'''sq_id = index_to_id(r_id, c_id)
							r_id += 1
							lon = np.array([c[0], c[1]])
							lat = np.array([r[0], r[1]])
							x, y = proj.lonlat_to_screen(lon, lat)
							w = x.max() - x.min()
							h = y.max() - y.min()
							w = np.ceil(w / 2) * 2
							h = np.ceil(h / 2) * 2
							l = max(w, h)
							n_x = np.array([x.min(), x.min() + l, x.min() + l, x.min()])
							n_y = np.array([y.min() + l, y.min() + l, y.min(), y.min()])
							self.painter.set_color([0, 0, 0])
							self.painter.linestrip(n_x, n_y, closed=True)
							self.hotspots.add_poly(n_x, n_y, self.keywords[name][sq_id])
							self.painter.set_color('b')'''
							lon_t.append(self.tweets[0].position['lon'])
							lat_t.append(self.tweets[0].position['lat'])
							xvt, yvt = proj.lonlat_to_screen(np.array(lon_t), np.array(lat_t))
							self.painter.circle_filled(self.tweets[0].position['lon'], self.tweets[0].position['lat'], 10)
							#self.painter.points(xvt, yvt, 10, True)

	def draw(self, proj, mouse_x, mouse_y, ui_manager):
		self.painter.batch_draw()
		picked = self.hotspots.pick(mouse_x, mouse_y)
		if picked:
			ui_manager.tooltip(picked)

	def update_tweet(self, tweets):
		self.tweets = []
		self.tweets.append(tweets)

	def update_keywords(self, keywords):
		self.keywords = keywords
