'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

    Copyright Paraskevopoulos, Pellegrini, Palpanas.


    This software cannot be used for commercial purposes.

    You may use the software as is, only for research purposes.
    You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

    Pavlos Paraskevopoulos, Themis Palpanas.
    Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
    http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

    Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
    http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

from geoplotlib.layers import BaseLayer
from geoplotlib.core import BatchPainter
from geoplotlib.utils import BoundingBox
from geoplotlib.layers import HotspotManager


class TestPointsLayer(BaseLayer):
    def __init__(self, data, color=None, point_size=2, f_tooltip=None, rounded=True, communicator=None):
        """Create a dot density map

        :param data: data access object
        :param color: color
        :param point_size: point size
        :param f_tooltip: function to return a tooltip string for a point
        """
        self.data = data
        self.color = color
        if self.color is None:
            self.color = [0, 0, 255, 220]
        self.point_size = point_size
        self.f_tooltip = f_tooltip
        self.rounded = rounded
        self.communicator = communicator
        self.hotspots = HotspotManager()

    def invalidate(self, proj):
        self.painter = BatchPainter()
        try:
            x, y = proj.lonlat_to_screen(self.data['lon'], self.data['lat'])
            if self.f_tooltip:
                for i in range(0, len(x)):
                    record = {k: self.data[k][i] for k in self.data.keys()}
                    self.hotspots.add_rect(x[i] - self.point_size, y[i] - self.point_size,
                                           2 * self.point_size, 2 * self.point_size,
                                           self.f_tooltip(record))
            self.painter.set_color(self.color)
            self.painter.points(x, y, 2 * self.point_size, self.rounded)
        except:
            print "No test tweets for this timeslot."

    def draw(self, proj, mouse_x, mouse_y, ui_manager):
        self.painter.batch_draw()
        picked = self.hotspots.pick(mouse_x, mouse_y)
        if picked:
            ui_manager.tooltip(picked)

    def update_data(self, data):
        self.data = data

    def bbox(self):
        return BoundingBox.from_points(lons=self.data['lon'], lats=self.data['lat'])

    def on_key_release(self, key, modifiers):
        self.communicator.on_key_release(key, "test")
