'''
    TweeLoc is created by

    Pavlos Paraskevopoulos:
        George Mason University
        LIPADE, Paris Descartes University
        pparaske@gmu.edu

    Giovanni Pellegrini:
        University of Trento
        giovanni.pellegrini@unitn.it

    Themis Palpanas:
        Paris Descartes University
        themis@mi.parisdescartes.fr

    Copyright Paraskevopoulos, Pellegrini, Palpanas.


    This software cannot be used for commercial purposes.

    You may use the software as is, only for research purposes.
    You should acknowledge the authors by referencing the following two papers (the first describes the method and algorithms; the second describes the system and visualizations):

    Pavlos Paraskevopoulos, Themis Palpanas.
    Where has this Tweet Come From? Fast and Fine-Grained Geolocalisation of Non-Geotagged Tweets. Social Network Analysis and Mining (SNAM) Journal 6(1), pp 1-16, 2016.
    http://www.mi.parisdescartes.fr/~themisp/publications/snam16.pdf

    Pavlos Paraskevopoulos, Giovanni Pellegrini, Themis Palpanas. TweeLoc: A System for Geolocalizing Tweets at Fine-Grain. International Conference on Data Mining (ICDM), New Orleans, LA, USA, November 2017.
    http://www.mi.parisdescartes.fr/~themisp/publications/icdm17-tweeloc.pdf
'''

from datetime import datetime
import re


class Tweet:
    def __init__(self, id, lang, time, text, position):
        self.id = id
        self.time = time
        self.text = text
        self.position = position
        self.language = lang

    def toJson(self):
        return "{\"created_at\":\"" + self.time.__str__() + \
               "\",\"id\":" + self.id + \
               ",\"id_str\":\"" + self.id + \
               "\",\"lang\":\"" + self.language + \
               "\",\"text\":\"" + self.text + \
               "\",\"coordinates\":[" + str(self.position['lon']) + "," + str(self.position['lat']) + "]}"


MASK = pow(2, 16) - 1


def index_to_id(r, c):
    index_r = (r & MASK) << 16
    index_c = c & MASK
    id = index_r | index_c
    return id


def parse_tweet_time(tweet_time):
    # extract the parts of the time from the string attribute "created_at" of the tweet
    new_tweet_time = datetime.strptime(tweet_time, "%Y-%m-%d %H:%M:%S")

    return new_tweet_time


def parse_tweet_position(tweet_position):
    return {'lon': tweet_position[0], 'lat': tweet_position[1]}


def parse_tweet_text(tweet_text):
    # text = tweet_text.encode('ascii', 'ignore')
    text_no_links = re.sub('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', '',
                           tweet_text)
    parsed_text = re.sub(r'[^\w]', ' ', text_no_links)
    parsed_text = parsed_text.lower()
    return parsed_text.lower()


def parse_strptime_strftime(time):
    return datetime.strftime(time, '%Y-%m-%d %H:%M:%S')
